# Configure Python bindings for one component library.
# Called from function MakeLib.

function(SwigLib name lib swigtmpdir)

    # TODO: Why BORNAGAIN_PYTHON is needed for Swig API?
    if(NOT BORNAGAIN_PYTHON)
        message(FATAL_ERROR "Function SwigLib called though BORNAGAIN_PYTHON=false")
    endif()

    # configure SWIG runtime (only once)
    if(NOT TARGET swig_runtime)
        message(STATUS "SwigLib: configure SWIG runtime")

        if(CONFIGURE_BINDINGS)
            add_custom_command(
                OUTPUT ${AUTO_WRAP_DIR}/swig_runtime.h
                COMMAND ${SWIG_EXECUTABLE} -c++;-python;-external-runtime;${AUTO_WRAP_DIR}/swig_runtime.h
                DEPENDS ${swig_dependencies} ${include_files}
            )
        endif()

        add_custom_target(swig_runtime DEPENDS ${AUTO_WRAP_DIR}/swig_runtime.h)

        if(BA_CPP_API)
            install(FILES ${AUTO_WRAP_DIR}/swig_runtime.h
                DESTINATION ${destination_include}/Wrap COMPONENT Headers)
        endif()
    endif()

    message(STATUS "SwigLib ${name}: ${lib} ${swigtmpdir}")

    if(CONFIGURE_BINDINGS)
        file(MAKE_DIRECTORY ${swigtmpdir})

        # static (manually written) input files
        set(swig_dependencies
            ${SWIG_DIR}/lib${lib}.i
            ${SWIG_DIR}/deprecation.i
            ${SWIG_DIR}/warnings.i
            ${SWIG_DIR}/ignoreSample.i
        )

        foreach(FNAM ${swig_dependencies})
            if(NOT EXISTS ${FNAM})
                message(FATAL_ERROR "Could NOT find SWIG input ${FNAM}")
            endif()
        endforeach()

        # Run Swig.
        #     Please keep -Werror, in order not to overlook critical warnings.
        #     Dispensable warnings are disabled in Wrap/Swig/warnings.i.
        #     Joachim, oct20.
        set(SWIG_FLAGS "-c++;-python;-Werror;-o;${AUTO_WRAP_DIR}/lib${lib}_wrap.cpp;-outdir;${swigtmpdir}"
            ";-I${LibHeinz_INCLUDE_DIR}"
            ";-I${CMAKE_SOURCE_DIR};-I${CMAKE_BINARY_DIR}/inc")

        add_custom_target(lib${lib}_py_wrappers ALL
            BYPRODUCTS ${swigtmpdir}/lib${lib}.py
                   ${AUTO_WRAP_DIR}/lib${lib}_wrap.h
                   ${AUTO_WRAP_DIR}/lib${lib}_wrap.cpp
                   ${AUTO_WRAP_DIR}/lib${lib}.py
            # produce the Python wrapper
            COMMAND ${SWIG_EXECUTABLE} ${SWIG_FLAGS} ${SWIG_DIR}/lib${lib}.i
            # amend the Python wrapper
            COMMAND ${Python3_EXECUTABLE} ${SWIG_DIR}/tweaks.py
                    ${swigtmpdir}/lib${lib}.py
                    ${AUTO_WRAP_DIR}/lib${lib}.py
            DEPENDS ${swig_dependencies} ${include_files}
            COMMENT "SWIG: build Python wrapper ${AUTO_WRAP_DIR}/lib${lib}.py"
            )

        string(REPLACE ";" " " SWIG_FLAGS_STR "${SWIG_FLAGS}")
        set(SWIG_FLAGS_STR "${SWIG_FLAGS_STR}" CACHE INTERNAL "")
        set(SWIG_DIR "${SWIG_DIR}" CACHE INTERNAL "")
    else()
        # assumes that lib${lib}_wrap.h, lib${lib}_wrap.cpp and lib${lib}.py
        # are already present in ${AUTO_WRAP_DIR}
        add_custom_target(lib${lib}_py_wrappers ALL
            DEPENDS ${AUTO_WRAP_DIR}/lib${lib}_wrap.h
            ${AUTO_WRAP_DIR}/lib${lib}_wrap.cpp
            ${AUTO_WRAP_DIR}/lib${lib}.py
        )
    endif()

    # include Python and Numpy headers and link to Python shared library
    target_include_directories(${lib} PRIVATE ${Python3_INCLUDE_DIRS} ${Python3_NumPy_INCLUDE_DIRS})
    target_link_libraries(${lib} PRIVATE ${Python3_LIBRARIES})

    set_target_properties(${lib} PROPERTIES
        _LIBRARY_PY "${AUTO_WRAP_DIR}/lib${lib}.py"
        )

    install(FILES "${AUTO_WRAP_DIR}/lib${lib}.py"
        DESTINATION ${destination_python} COMPONENT Libraries)

endfunction()
